"use strict";
exports.__esModule = true;
var EBuilderVerifyMode;
(function (EBuilderVerifyMode) {
    EBuilderVerifyMode[EBuilderVerifyMode["CASE"] = 0] = "CASE";
    EBuilderVerifyMode[EBuilderVerifyMode["CONTENT"] = 1] = "CONTENT";
    EBuilderVerifyMode[EBuilderVerifyMode["FOR"] = 2] = "FOR";
    EBuilderVerifyMode[EBuilderVerifyMode["HEADER"] = 3] = "HEADER";
    EBuilderVerifyMode[EBuilderVerifyMode["IF"] = 4] = "IF";
    EBuilderVerifyMode[EBuilderVerifyMode["INDENT"] = 5] = "INDENT";
    EBuilderVerifyMode[EBuilderVerifyMode["PRINT"] = 6] = "PRINT";
    EBuilderVerifyMode[EBuilderVerifyMode["SWITCH"] = 7] = "SWITCH";
    EBuilderVerifyMode[EBuilderVerifyMode["TRY"] = 8] = "TRY";
})(EBuilderVerifyMode || (EBuilderVerifyMode = {}));
var Builder = (function () {
    function Builder(props) {
        this.props = props;
        this.bespokes = [];
        this.built = "";
        this.forLevel = 0;
        this.identifiers = [];
        this.ifLevel = 0;
        this.indentation = 0;
        this.switchCaseLevel = 0;
        this.tryLevel = 0;
    }
    Builder["new"] = function (props) {
        return new Builder(props);
    };
    Builder.prototype.add = function (content) {
        return this._add(content, false, false, false);
    };
    Builder.prototype.addThenNewline = function (content) {
        return this._add(content, false, true, false);
    };
    Builder.prototype.await = function (content) {
        return this._add(content, true, false, false);
    };
    Builder.prototype.awaitThenNewline = function (content) {
        return this._add(content, true, true, false);
    };
    Builder.prototype["case"] = function (check) {
        this.switchCaseLevel++;
        this.verify(EBuilderVerifyMode.CASE);
        return this
            .addThenNewline("case " + check + ": {")
            .indent();
    };
    Builder.prototype["catch"] = function (name) {
        this.verify(EBuilderVerifyMode.TRY);
        this.unindent();
        if (name === undefined) {
            this.addThenNewline("} catch {");
        }
        else {
            this.addThenNewline("} catch(" + name + ") {");
        }
        return this.indent();
    };
    Builder.prototype["default"] = function () {
        this.switchCaseLevel++;
        this.verify(EBuilderVerifyMode.CASE);
        return this
            .addThenNewline("default: {")
            .indent();
    };
    Builder.prototype["else"] = function () {
        this.verify(EBuilderVerifyMode.IF);
        return this
            .unindent()
            .addThenNewline("} else {")
            .indent();
    };
    Builder.prototype.elseIf = function (check) {
        this.verify(EBuilderVerifyMode.IF);
        return this
            .unindent()
            .addThenNewline("} else if (" + check + ") {")
            .indent();
    };
    Builder.prototype.endCase = function () {
        this.verify(EBuilderVerifyMode.CASE);
        this.switchCaseLevel--;
        return this
            .addThenNewline("break;")
            .unindent()
            .addThenNewline("}");
    };
    Builder.prototype.endFor = function () {
        this.verify(EBuilderVerifyMode.FOR);
        this.forLevel--;
        return this
            .unindent()
            .addThenNewline("}");
    };
    Builder.prototype.endIf = function () {
        this.verify(EBuilderVerifyMode.IF);
        this.ifLevel--;
        return this
            .unindent()
            .addThenNewline("}");
    };
    Builder.prototype.endSwitch = function () {
        this.verify(EBuilderVerifyMode.SWITCH);
        this.switchCaseLevel--;
        return this
            .unindent()
            .addThenNewline("}");
    };
    Builder.prototype.endTry = function () {
        this.verify(EBuilderVerifyMode.TRY);
        this.tryLevel--;
        return this
            .unindent()
            .addThenNewline("}");
    };
    Builder.prototype.ensureOnNewline = function () {
        return this._ensureOnNewline(1);
    };
    Builder.prototype.ensureOnNewlineAfterEmptyline = function () {
        return this._ensureOnNewline(2);
    };
    Builder.prototype["finally"] = function () {
        this.verify(EBuilderVerifyMode.TRY);
        return this
            .unindent()
            .addThenNewline("} finally {")
            .indent();
    };
    Builder.prototype["for"] = function (check) {
        this.forLevel++;
        this.verify(EBuilderVerifyMode.FOR);
        return this
            .addThenNewline("for (" + check + ") {")
            .indent();
    };
    Builder.prototype.getBespokes = function () {
        return this.bespokes;
    };
    Builder.prototype.getIdentifiers = function () {
        return this.identifiers;
    };
    Builder.prototype.getName = function () {
        return this.props.name;
    };
    Builder.prototype.getPath = function () {
        return this.props.path;
    };
    Builder.prototype["if"] = function (check) {
        this.ifLevel++;
        this.verify(EBuilderVerifyMode.IF);
        return this
            .addThenNewline("if (" + check + ") {")
            .indent();
    };
    Builder.prototype.indent = function () {
        return this._indent(1);
    };
    Builder.prototype.print = function () {
        this.verify(EBuilderVerifyMode.PRINT);
        var b = Builder["new"](this.props);
        if (this.header !== undefined) {
            b.built = this.header;
        }
        b.ensureOnNewlineAfterEmptyline();
        b.built += this.built;
        while (b.built.endsWith("\n\n")) {
            b.built = b.built.slice(0, b.built.length - 1);
        }
        return b.built;
    };
    Builder.prototype["return"] = function (content) {
        return this._add(content, false, true, true);
    };
    Builder.prototype.returnAwait = function (content) {
        return this._add(content, true, true, true);
    };
    Builder.prototype.setHeader = function (header) {
        this.verify(EBuilderVerifyMode.HEADER);
        this.header = header;
        return this;
    };
    Builder.prototype["switch"] = function (check) {
        this.switchCaseLevel++;
        this.verify(EBuilderVerifyMode.SWITCH);
        return this
            .addThenNewline("switch (" + check + ") {")
            .indent();
    };
    Builder.prototype["try"] = function () {
        this.tryLevel++;
        this.verify(EBuilderVerifyMode.TRY);
        return this
            .addThenNewline("try {")
            .indent();
    };
    Builder.prototype.unindent = function () {
        return this._indent(-1);
    };
    Builder.prototype.withBespokes = function () {
        var bespokes = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            bespokes[_i] = arguments[_i];
        }
        this.bespokes = this.bespokes.concat(bespokes);
        return this;
    };
    Builder.prototype.withIdentifiers = function () {
        var identifiers = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            identifiers[_i] = arguments[_i];
        }
        this.identifiers = this.identifiers.concat(identifiers);
        return this;
    };
    Builder.prototype._add = function (content, isAsync, newlineAfter, isReturn) {
        this.verify(EBuilderVerifyMode.CONTENT, content);
        if (this.built.length > 0 && this.built.endsWith("\n")) {
            this.built += "  ".repeat(this.indentation);
        }
        if (isReturn) {
            this.built += "return ";
        }
        if (isAsync) {
            this.built += "await ";
        }
        this.built += content;
        if (newlineAfter) {
            this.built += "\n";
        }
        return this;
    };
    Builder.prototype._ensureOnNewline = function (newLinesWanted) {
        if (this.built.length === 0) {
            return this;
        }
        var newLinesFound = 0;
        for (var idx = 0; idx < newLinesWanted; idx++) {
            if (this.built.charAt(this.built.length - idx - 1) === "\n") {
                newLinesFound++;
            }
            else {
                break;
            }
        }
        this.built += "\n".repeat(newLinesWanted - newLinesFound);
        return this;
    };
    Builder.prototype._indent = function (delta) {
        this.indentation += delta;
        this.verify(EBuilderVerifyMode.INDENT);
        return this;
    };
    Builder.prototype.verify = function (mode, content) {
        if (mode === EBuilderVerifyMode.CASE) {
            if (this.switchCaseLevel < 1 || this.switchCaseLevel % 2 !== 0) {
                throw new Error("Not inside case statement");
            }
        }
        if (mode === EBuilderVerifyMode.CONTENT) {
            if (content === undefined) {
                throw new Error("Unreachable");
            }
            if (content === "") {
                throw new Error("Unexpected empty string");
            }
            if (content.includes("\n")) {
                throw new Error("Unexpected newline");
            }
            if (content.includes("\t")) {
                throw new Error("Unexpected tab");
            }
            if (content.includes("await")) {
                throw new Error("Unexpected await");
            }
            if (content.includes("return")) {
                throw new Error("Unexpected return");
            }
        }
        if (mode === EBuilderVerifyMode.HEADER) {
            if (this.header !== undefined) {
                throw new Error("Cannot set header twice");
            }
        }
        if (mode === EBuilderVerifyMode.INDENT) {
            if (!this.built.endsWith("\n")) {
                throw new Error("Cannot change indentation without being on newline");
            }
            if (this.indentation < 0) {
                throw new Error("Cannot unindent past 0");
            }
        }
        if (mode === EBuilderVerifyMode.PRINT) {
            if (this.indentation !== 0) {
                throw new Error("Cannot print without zeroed indentation");
            }
            if (this.forLevel !== 0) {
                throw new Error("Cannot print without completing for statements");
            }
            if (this.ifLevel !== 0) {
                throw new Error("Cannot print without completing if statements");
            }
            if (this.switchCaseLevel !== 0) {
                throw new Error("Cannot print without completing switch statements");
            }
            if (this.tryLevel !== 0) {
                throw new Error("Cannot print without completing try statements");
            }
        }
        if (mode === EBuilderVerifyMode.FOR) {
            if (this.forLevel < 1) {
                throw new Error("Not inside for statement");
            }
        }
        if (mode === EBuilderVerifyMode.IF) {
            if (this.ifLevel < 1) {
                throw new Error("Not inside if statement");
            }
        }
        if (mode === EBuilderVerifyMode.SWITCH) {
            if (this.switchCaseLevel < 1 || this.switchCaseLevel % 2 !== 1) {
                throw new Error("Not inside switch statement");
            }
        }
        if (mode === EBuilderVerifyMode.TRY) {
            if (this.tryLevel < 1) {
                throw new Error("Not inside try statement");
            }
        }
    };
    return Builder;
}());
exports.Builder = Builder;
