"use strict";
exports.__esModule = true;
var bespoke_1 = require("./bespoke");
var class_1 = require("./class");
var function_1 = require("./function");
var import_1 = require("./import");
var interface_1 = require("./interface");
var method_1 = require("./method");
var module_1 = require("./module");
var type_1 = require("./type");
function JestCall(props) {
    return function (builder) {
        builder
            .addThenNewline(props.functionName + "(")
            .indent();
        if (props.testName !== undefined) {
            builder.addThenNewline("\"" + props.testName + "\",");
        }
        builder
            .addThenNewline("async (): Promise<void> => {")
            .indent();
        if (props.testName !== undefined) {
            bespoke_1.Bespoke["new"]({ name: props.testName })
                .run(builder);
        }
        else {
            bespoke_1.Bespoke["new"]({ name: props.functionName })
                .run(builder);
        }
        builder
            .unindent()
            .addThenNewline("},")
            .unindent()
            .addThenNewline(");");
    };
}
function Jest(props) {
    var bespokeImport = bespoke_1.Bespoke["new"]({
        name: "imports"
    });
    var beforeAll = JestCall({
        functionName: "beforeAll"
    });
    var afterAll = JestCall({
        functionName: "afterAll"
    });
    var beforeEach = JestCall({
        functionName: "beforeEach"
    });
    var afterEach = JestCall({
        functionName: "afterEach"
    });
    var testRenders = props.tests.map(function (test) { return JestCall({
        functionName: "test",
        testName: test
    }); });
    return module_1.Module["new"]({
        content: [
            bespokeImport,
            beforeAll,
            afterAll,
            beforeEach,
            afterEach
        ].concat(testRenders),
        destination: props.destination
    });
}
exports.Jest = Jest;
function ReactConstructorCall() {
    return function (builder) {
        builder
            .addThenNewline("super(props);")
            .addThenNewline("this.state = {")
            .indent();
        bespoke_1.Bespoke["new"]({ name: "state" })
            .run(builder);
        builder
            .unindent()
            .addThenNewline("};");
    };
}
function RelayContainerCall(props) {
    return function (builder) {
        builder
            .addThenNewline("const _" + props.name + ": React.ComponentType" + props.propOnlyExtendsType + " = " + props.relayType + "(")
            .indent()
            .addThenNewline("__" + props.name + ",");
        bespoke_1.Bespoke["new"]({
            name: "relay"
        })
            .run(builder);
        builder
            .unindent()
            .addThenNewline(");");
    };
}
var ERelayType;
(function (ERelayType) {
    ERelayType["FRAGMENT"] = "createFragmentContainer";
    ERelayType["PAGINATION"] = "createPaginationContainer";
    ERelayType["REFETCH"] = "createRefetchContainer";
})(ERelayType = exports.ERelayType || (exports.ERelayType = {}));
function React(props) {
    var extendsType = "";
    var propOnlyExtendsType = "";
    var content = [
        import_1.Import["new"]({
            name: "react",
            withAllAs: "React"
        }),
        bespoke_1.Bespoke["new"]({
            name: "imports"
        }),
    ];
    if (props.props === undefined) {
        content = content.concat([
            function_1.Function.Sync.newExported({
                content: [
                    bespoke_1.Bespoke["new"]({
                        name: "render"
                    }),
                ],
                inTypes: [],
                name: "_" + props.name,
                outType: type_1.Type.Anonymous["new"]({
                    type: "JSX.Element"
                })
            }),
        ]);
    }
    else {
        var propsName = "I" + props.name + "Props";
        extendsType += "<" + propsName;
        if (props.relayType === undefined) {
            propOnlyExtendsType += "<" + propsName + ">";
        }
        else {
            propOnlyExtendsType += "<MappedFragmentProps<RemoveRelayProp<" + propsName + ">>>";
            props.props = props.props.concat([
                type_1.Type.Required["new"]({
                    name: "data",
                    type: props.name + "Query"
                }),
            ]);
        }
        content = content.concat([
            interface_1.Interface.newExported({
                name: propsName,
                types: props.props
            }),
        ]);
        var constructor = [];
        if (props.state !== undefined) {
            var stateName = "I" + props.name + "State";
            extendsType += ", " + stateName + ">";
            content = content.concat([
                interface_1.Interface.newExported({
                    name: stateName,
                    types: props.state
                }),
            ]);
            constructor = [
                method_1.Method.Instance.Public.newConstructor({
                    content: [
                        ReactConstructorCall(),
                    ],
                    inTypes: [
                        type_1.Type.Argument["new"]({
                            name: "props",
                            type: propsName
                        }),
                    ]
                }),
            ];
        }
        else {
            extendsType += ">";
        }
        var classContent = constructor.concat([
            method_1.Method.Instance.Public.newSync({
                content: [
                    bespoke_1.Bespoke["new"]({
                        name: "render"
                    }),
                ],
                inTypes: [],
                name: "render",
                outType: type_1.Type.Anonymous["new"]({
                    type: "JSX.Element"
                })
            }),
            bespoke_1.Bespoke["new"]({
                name: "implementation"
            }),
        ]);
        if (props.relayType === undefined) {
            content = content.concat([
                class_1.Class.Concrete.newInternal({
                    content: classContent,
                    "extends": "React.Component" + extendsType,
                    name: "_" + props.name
                }),
            ]);
            if (props.relayMutation === true) {
                content = content.concat([
                    import_1.Import["new"]({
                        name: "react-relay",
                        "with": ["commitMutation", "graphql"]
                    }),
                ]);
            }
        }
        else {
            content = content.concat([
                class_1.Class.Concrete.newInternal({
                    content: classContent,
                    "extends": "React.Component" + extendsType,
                    name: "__" + props.name
                }),
            ]);
            var relayImports = ["graphql", props.relayType];
            if (props.relayMutation === true) {
                relayImports = relayImports.concat(["commitMutation"]);
            }
            relayImports = relayImports.concat(["MappedFragmentProps", "RemoveRelayProp"]);
            content = content.concat([
                import_1.Import["new"]({
                    name: "react-relay",
                    "with": relayImports
                }),
                RelayContainerCall({
                    name: props.name,
                    relayType: props.relayType,
                    propOnlyExtendsType: propOnlyExtendsType
                }),
            ]);
        }
    }
    content = content.concat([
        function (builder) {
            builder.addThenNewline("export { _" + props.name + " as " + props.name + " };");
        },
    ]);
    return module_1.Module["new"]({
        content: content,
        destination: props.destination
    });
}
exports.React = React;
