import { Builder } from "./builder";
import { NamedRenderer, TRenderer } from "./renderer";
import { Type } from "./type";
export declare namespace Method {
    enum E {
        PRIVATE = "private",
        PROTECTED = "protected",
        PUBLIC = "public"
    }
    interface IBase {
        readonly content?: TRenderer[];
        readonly templates?: string[];
    }
    interface IConstructor extends IBase {
        readonly inTypes: Array<Type.Argument | Type.FromProperty>;
    }
    interface INormal extends IBase {
        readonly inTypes: Type.Argument[];
        readonly name: string;
        readonly outType: Type.Anonymous;
    }
    type T = IConstructor | INormal;
    abstract class Base extends NamedRenderer {
        private readonly props;
        private readonly async;
        private readonly isStatic;
        private readonly kind;
        protected constructor(props: T, async: boolean, isStatic: boolean, kind: E);
        protected render(builder: Builder): void;
        protected verify(builder: Builder): void;
    }
    abstract class BaseInstance extends Base {
    }
    namespace Instance {
        class Private extends BaseInstance {
            static newAsync(props: INormal): Private;
            static newConstructor(props: IConstructor): Private;
            static newSync(props: INormal): Private;
        }
        class Protected extends BaseInstance {
            static newAsync(props: INormal): Protected;
            static newConstructor(props: IConstructor): Protected;
            static newSync(props: INormal): Protected;
        }
        class Public extends BaseInstance {
            static newAsync(props: INormal): Public;
            static newConstructor(props: IConstructor): Public;
            static newSync(props: INormal): Public;
        }
    }
    abstract class BaseStatic extends Base {
    }
    namespace Static {
        class Private extends BaseStatic {
            static newAsync(props: INormal): Private;
            static newSync(props: INormal): Private;
        }
        class Protected extends BaseStatic {
            static newAsync(props: INormal): Protected;
            static newSync(props: INormal): Protected;
        }
        class Public extends BaseStatic {
            static newAsync(props: INormal): Public;
            static newSync(props: INormal): Public;
        }
    }
}
