"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
var crypto_1 = require("crypto");
var import_1 = require("./import");
var renderer_1 = require("./renderer");
var headerTemplateWithoutBespoke = "/**\n * This file is fully generated; do not manually edit.\n *\n *\n * SOURCE<<@0>>\n * SIGNED<<@1>>\n */";
var headerTemplateWithBespoke = "/**\n * This file is partially generated; only edit bespoke sections.\n *\n * SOURCE<<@0>>\n * BESPOKE<<@1>>\n * SIGNED<<@2>>\n */";
var Module = (function (_super) {
    __extends(Module, _super);
    function Module(props) {
        var _this = _super.call(this) || this;
        _this.props = props;
        return _this;
    }
    Module["new"] = function (props) {
        return new Module(props);
    };
    Module.getHash = function (builder) {
        var hash = crypto_1.createHash("SHA512");
        hash.update(builder.print());
        return hash.digest("base64");
    };
    Module.renderImports = function (builder, imports) {
        Module.renderImportSection(builder, imports
            .filter(function (i) { return i.kind() === import_1.EImportKind.RAW; }));
        builder.ensureOnNewlineAfterEmptyline();
        Module.renderImportSection(builder, imports
            .filter(function (i) { return i.kind() === import_1.EImportKind.GLOBAL; }));
        builder.ensureOnNewlineAfterEmptyline();
        Module.renderImportSection(builder, imports
            .filter(function (i) { return i.kind() === import_1.EImportKind.LOCAL; }));
        builder.ensureOnNewlineAfterEmptyline();
    };
    Module.renderImportSection = function (builder, imports) {
        imports.forEach(function (i) {
            i.run(builder);
        });
    };
    Module.renderNonImports = function (builder, nonImports) {
        nonImports
            .forEach(function (r) {
            builder.ensureOnNewlineAfterEmptyline();
            Module.genericRenderer(r)(builder);
        });
    };
    Module.prototype.destination = function () {
        return this.props.destination;
    };
    Module.prototype.render = function (builder) {
        var imports = this.props.content
            .filter(function (i) { return i instanceof import_1.Import; })
            .sort(function (a, b) { return a.compare(b); });
        Module.renderImports(builder, imports);
        var nonImports = this.props.content
            .filter(function (i) { return !(i instanceof import_1.Import); });
        Module.renderNonImports(builder, nonImports);
        var bespokes = builder.getBespokes();
        var header = "";
        if (bespokes.length > 0) {
            header = headerTemplateWithBespoke
                .replace("@0", builder.getPath() + "::" + builder.getName())
                .replace("@1", bespokes.join(", "))
                .replace("@2", Module.getHash(builder));
        }
        else {
            header = headerTemplateWithoutBespoke
                .replace("@0", builder.getPath() + "::" + builder.getName())
                .replace("@1", Module.getHash(builder));
        }
        builder.setHeader(header);
    };
    Module.prototype.verify = function (builder) {
        this.verifyUniqueBespoke(builder);
        this.verifyUniqueIdentifiers(builder);
    };
    Module.prototype.verifyUniqueBespoke = function (builder) {
        var bespokeArray = builder.getBespokes();
        var bespokeSet = new Set(bespokeArray);
        if (bespokeSet.size < bespokeArray.length) {
            throw new Error("Duplicated Bespoke Sections");
        }
    };
    Module.prototype.verifyUniqueIdentifiers = function (builder) {
        var identifierArray = builder.getIdentifiers();
        var identifierSet = new Set(identifierArray);
        if (identifierSet.size < identifierArray.length) {
            throw new Error("Duplicated Identifier Names");
        }
    };
    return Module;
}(renderer_1.NamedRenderer));
exports.Module = Module;
