import { Builder } from "./builder";
import { Method } from "./method";
import { Property } from "./property";
import { NamedRenderer } from "./renderer";
export declare namespace Type {
    interface IAnonymousSimple {
        readonly type: string;
    }
    interface IAnonymousUnion {
        readonly types: string[];
    }
    interface INamedSimple extends IAnonymousSimple {
        readonly name: string;
    }
    interface INamedUnion extends IAnonymousUnion {
        readonly name: string;
    }
    interface IArgumentSimple extends INamedSimple {
        readonly default?: string;
    }
    interface IArgumentUnion extends INamedUnion {
        readonly default?: string;
    }
    interface IMethod {
        readonly method: Method.BaseInstance;
    }
    interface IProperty {
        readonly property: Property.BaseInstance;
    }
    type TAnonymous = IAnonymousSimple | IAnonymousUnion;
    type TArgument = IArgumentSimple | IArgumentUnion;
    type TMethod = IMethod;
    type TNamed = INamedSimple | INamedUnion;
    type TProperty = IProperty;
    type T = TAnonymous | TArgument | TMethod | TNamed | TProperty;
    abstract class Base extends NamedRenderer {
        private readonly props;
        private readonly optional;
        protected constructor(props: T, optional: boolean);
        protected render(builder: Builder): void;
        protected verify(builder: Builder): void;
    }
    class Anonymous extends Base {
        static new(props: TAnonymous): Anonymous;
    }
    class Argument extends Base {
        static new(props: TArgument): Argument;
    }
    class FromMethod extends Base {
        static new(props: TMethod): FromMethod;
    }
    class FromProperty extends Base {
        static new(props: TProperty): FromProperty;
    }
    class Optional extends Base {
        static new(props: TNamed): Optional;
    }
    class Required extends Base {
        static new(props: TNamed): Required;
    }
}
